#==============================================================================
# package      DE_EPAGES::CreditCheck::API::TLE::CreditCheckProviderHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides TLE functions for CreditCheck
#==============================================================================
package DE_EPAGES::CreditCheck::API::TLE::CreditCheckProviderHandler;

use strict;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Log qw( GetLog );

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = $Package->new();
# example      my $Handler  = $Package->new();
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     #HideCreditCheckProviderByDeprecation
# state        public
#----------------------------------------------------------------------------------------
# syntax       #HideCreditCheckProviderByDeprecation( $CreditCheckProviderConfig )
# example      #HideCreditCheckProviderByDeprecation( #CreditCheckProviderConfig )
#----------------------------------------------------------------------------------------
# description  Returns 1 if credit check provider was deprecated and there is no credit
#               check provider configuration defined or there is but not usable.
#               Otherwise returns 0;
#----------------------------------------------------------------------------------------
# input        #Processor  | processor | object
# input        #Params | parameter (CreditCheckProviderConfig) | array
# return       #Hide | 1 or 0 | bit
#========================================================================================
sub HideCreditCheckProviderByDeprecation {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($CreditCheckProvider, $CreditCheckProviderConfig) = @$aParams;

    return 0 unless ($CreditCheckProvider->isDeprecated());
    return 0 if (isValid($CreditCheckProviderConfig) && $CreditCheckProviderConfig->usableAtStorefront());
    return 1;
}

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               <\ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params)     = @_;
    GetLog->debug( 'CreditCheckProviderHandler.RegisterHandlerProc' );
    my $TLEProcessor = $Params->{'Processor'};
    my $Handler      = __PACKAGE__->new();
    $TLEProcessor->registerHandler( 'FunctionHandler', $Handler, 'HideCreditCheckProviderByDeprecation' );
    return;
}

1;