#========================================================================================
# package      DE_EPAGES::CreditCheck::t::unit::CreditCheckProvider
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for methods from CreditCheckProvider object
#               in package DE_EPAGES::CreditCheck::API::Object::CreditCheckProvider
#========================================================================================
package DE_EPAGES::CreditCheck::t::unit::CreditCheckProvider;
use base DE_EPAGES::Test::API::UnitTest::Test;

use strict;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use Test::More;

sub test {
    my $self = shift;

    $self->test_CreditCheckProvider_when_no_deprecation_date();
    $self->test_CreditCheckProvider_when_deprecation_date_in_the_future();
    $self->test_CreditCheckProvider_when_deprecation_date_in_the_past();

    done_testing();
}

sub test_CreditCheckProvider_when_no_deprecation_date {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $CreditCheckProvider = $self->_createSUT({ 'DeprecationDate' => undef });

    is($CreditCheckProvider->isDeprecated(), 0, "$SubTestName: isDeprecated method returns zero.");
}

sub test_CreditCheckProvider_when_deprecation_date_in_the_future {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $CurrentDateTime = GetCurrentDateTime();
    my $CreditCheckProvider = $self->_createSUT({ 'DeprecationDate' => $CurrentDateTime->add(days => 1) });

    is($CreditCheckProvider->isDeprecated(), 0, "$SubTestName: isDeprecated method returns zero.");
}

sub test_CreditCheckProvider_when_deprecation_date_in_the_past {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $CurrentDateTime = GetCurrentDateTime();
    my $CreditCheckProvider = $self->_createSUT({ 'DeprecationDate' => $CurrentDateTime->subtract(days => 1) });

    is($CreditCheckProvider->isDeprecated(), 1, "$SubTestName: isDeprecated method returns one.");
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($hSetup) = @_;

    my $hAttributes = {
        'DeprecationDate' => $hSetup->{'DeprecationDate'},
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::CreditCheck::API::Object::CreditCheckProvider',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();
