#========================================================================================
# package      DE_EPAGES::CreditCheck::t::unit::CreditCheckProviderHandler
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for API::TLE::CreditCheckProviderHandler
#========================================================================================
package DE_EPAGES::CreditCheck::t::unit::CreditCheckProviderHandler;
use base DE_EPAGES::Test::API::UnitTest::Test;

use strict;

use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::CreditCheck::API::TLE::CreditCheckProviderHandler;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Mockify;
use Test::Mockify::Matcher qw ( Object String );
use Test::Mockify::Sut;
use Test::More;


sub test {
    my $self = shift;

    $self->test_register();
    $self->test_HideCreditCheckProviderByDeprecation_when_no_deprecation_date();
    $self->test_HideCreditCheckProviderByDeprecation_when_deprecation_date_in_the_past_and_exists_config();
    $self->test_HideCreditCheckProviderByDeprecation_when_deprecation_date_in_the_past_and_no_config();

    done_testing();
}

sub test_register {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Processor = $self->_createProcessor();

    is(
        DE_EPAGES::CreditCheck::API::TLE::CreditCheckProviderHandler::RegisterHandlerProc({'Processor' => $Processor}),
        undef,
        "$SubTestName - prove that RegisterHandlerProc can be called"
    );

}

sub test_HideCreditCheckProviderByDeprecation_when_no_deprecation_date {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $CreditCheckProvider = $self->_createCreditCheckProvider({ 'DeprecationDate' => undef });
    my $aParams = [$CreditCheckProvider, undef];
    my $DummyProcessor;

    my $Result = $SUT->HideCreditCheckProviderByDeprecation($DummyProcessor,$aParams);
    is($Result, 0, "$SubTestName:-result is zero (it must not be hidden)");
}

sub test_HideCreditCheckProviderByDeprecation_when_deprecation_date_in_the_past_and_exists_config {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $CurrentDateTime = GetCurrentDateTime();
    my $CreditCheckProvider = $self->_createCreditCheckProvider({ 'DeprecationDate' => $CurrentDateTime->subtract(days => 2) });
    my $CreditCheckProviderConfig = $self->_createCreditCheckProviderConfig();
    my $aParams = [$CreditCheckProvider, $CreditCheckProviderConfig];
    my $DummyProcessor;

    my $Result = $SUT->HideCreditCheckProviderByDeprecation($DummyProcessor,$aParams);
    is($Result, 0, "$SubTestName:-result is zero (it must not be hidden)");
}

sub test_HideCreditCheckProviderByDeprecation_when_deprecation_date_in_the_past_and_no_config {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $CurrentDateTime = GetCurrentDateTime();
    my $CreditCheckProvider = $self->_createCreditCheckProvider({ 'DeprecationDate' => $CurrentDateTime->subtract(days => 2) });
    my $aParams = [$CreditCheckProvider, undef];
    my $DummyProcessor;

    my $Result = $SUT->HideCreditCheckProviderByDeprecation($DummyProcessor,$aParams);
    is($Result, 1, "$SubTestName:-result is one (it must be hidden)");
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createProcessor {
    my $self = shift;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::TLE::API::Processor' );
    $Mockify
        ->mock('registerHandler')
        ->when(
        String('FunctionHandler'),
        Object('DE_EPAGES::CreditCheck::API::TLE::CreditCheckProviderHandler'),
        String('HideCreditCheckProviderByDeprecation')
    )->thenReturnUndef();

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = Test::Mockify::Sut->new(
        'DE_EPAGES::CreditCheck::API::TLE::CreditCheckProviderHandler',
        $aParameterList
    );


    #
    # $Mockify->mockImported('DE_EPAGES::Object::API::Table::Currency', 'GetInfoCurrency')
    #     ->whenAny()
    #     ->thenReturn({'MinorUnit' => 3});



    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createCreditCheckProvider {
    my $self = shift;
    my ($hSetup) = @_;

    my $hAttributes = {
        'DeprecationDate' => $hSetup->{'DeprecationDate'},
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::CreditCheck::API::Object::CreditCheckProvider',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createCreditCheckProviderConfig {
    my $self = shift;

    my $hAttributes = {};

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::CreditCheck::API::Object::CreditCheckProviderConfig',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();

