#========================================================================================
# §package      DE_EPAGES::CrossSelling::API::CrossSellingPatch
# §state        public
#----------------------------------------------------------------------------------------
# §description  Implements JSON-PATCH (RFC 6902) for cross selling assignments.
#========================================================================================
package DE_EPAGES::CrossSelling::API::CrossSellingPatch;
use base DE_EPAGES::WebService::API::JSONPatch;

use DE_EPAGES::CrossSelling::API::CrossSelling qw ( AddCrossSelling FindCrossSelling );

use feature 'switch';
use strict;

## no critic (ProhibitUnusedPrivateSubroutines)



#========================================================================================
# §function     new
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       new( $Product );
#----------------------------------------------------------------------------------------
# §description  Constructor
#----------------------------------------------------------------------------------------
# §input        $Product | A Product | DE_EPAGES::Product::API::Object::Product
# §return       $self | self | DE_EPAGES::CrossSelling::API::CrossSellingPatch
#========================================================================================
sub new {
    my $class = shift;
    my ($Product) = @_;
    my $self  = bless {
        'Product' => $Product,
    }, $class;
    return $self;
}



sub _add {
    my $self = shift;
    my ($Path, $Value) = @_;

    my $Product = $self->{'Product'};

    my $Shop = $Product->get('Shop');
    my $Object;
    eval {
        $Object = $Shop->loadObjectByGuid($Value, 'Product');
    };
    return if $@ || ! defined $Object;

    my $Type = 'CrossSelling';
    AddCrossSelling($Product, [$Object], $Type, 0);

    return $Value;
}



sub _remove {
    my $self = shift;
    my ($Path, $Value) = @_;

    my $Product = $self->{'Product'};

    my $Shop = $Product->get('Shop');
    my $Object;
    eval {
        $Object = $Shop->loadObjectByGuid($Value, 'Product');
    };
    return if $@ || ! defined $Object;

    my $Type = 'CrossSelling';
    my $ExistingCrossSell = FindCrossSelling($Product, $Object, $Type );

    $ExistingCrossSell->delete if $ExistingCrossSell;

    return $Value;
}


1;
