#========================================================================================
# package      DE_EPAGES::DocdataPay::t::unit::CompiledClient
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::DocdataPay::t::unit::CompiledClient
#========================================================================================
package DE_EPAGES::DocdataPay::t::unit::CompiledClient;

use base DE_EPAGES::Core::API::Test;
use File::Spec;
use strict;

use Data::Dumper;
use DE_EPAGES::DocdataPay::API::SOAP::CompiledClient;
use DE_EPAGES::Test::Mock::EpagesObject;
use LWP::UserAgent;
use Readonly;
use Test::MockObject::Extends;
use Test::More;
use Test::Exception;
use XML::Compile::Transport::SOAPHTTP;

sub test {
    my $self = shift;
    $self->_test_create_compiled_client_positive();
    $self->_test_create_compiled_client_invalid_method();
    $self->_test_create_compiled_client_invalid_wsdl_location();
    done_testing();
}

sub _test_create_compiled_client_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $CompiledClient = $self->_getMockedCompiledClient();
    my $WebserviceCall = $CompiledClient->buildWSDL('create');
    is( ref($WebserviceCall), 'CODE', "$SubTestName - buildWSDL returns a function pointer when using create method." );

    $WebserviceCall = $CompiledClient->buildWSDL('capture');
    is( ref($WebserviceCall), 'CODE',
        "$SubTestName - buildWSDL returns a function pointer when using capture method." );

    $WebserviceCall = $CompiledClient->buildWSDL('status');
    is( ref($WebserviceCall), 'CODE', "$SubTestName - buildWSDL returns a function pointer when using status method." );

    return;
}

sub _test_create_compiled_client_invalid_method {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $CompiledClient = $self->_getMockedCompiledClient();

    throws_ok {
        do {
            local *STDERR= '';
            open STDERR, '>', File::Spec->devnull() or die "Can't redirect STDERR to /dev/null: $!";
            $CompiledClient->buildWSDL('undefined');
        }
    } (
        qr/no operation \`undefined\' for portType/,
        "$SubTestName - Throws an exception if you try to use an invalid method."
    );

    return;
}

sub _test_create_compiled_client_invalid_wsdl_location {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $CompiledClient = $self->_getMockedCompiledClient();

    throws_ok {
        do {
            local *STDERR= '';
            open STDERR, '>', File::Spec->devnull() or die "Can't redirect STDERR to /dev/null: $!";
            $CompiledClient->buildWSDL( 'create', '' );
        }
    }
    ( 'DE_EPAGES::Core::API::Error', "$SubTestName - Throws an exception if WSDL location is invalid." );

    return;
}

sub _getMockedCompiledClient {
    my $self = shift;

    my $CompiledClient       = DE_EPAGES::DocdataPay::API::SOAP::CompiledClient->new(1);
    my $MockedCompiledClient = Test::MockObject::Extends->new($CompiledClient);

    $MockedCompiledClient->mock(
        '_LoadRootObject',
        sub {
            my $hAttributes = {
                'DocdataPay_SOAPTestURL' => 'https://test.docdatapayments.com/ps/services/paymentservice/1_2',
                'DocdataPay_SOAPURL'     => ''
            };
            my $MockedSystem =
              DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::Site', $hAttributes );
            return $MockedSystem;
        }
    );

    $MockedCompiledClient->mock(
        '_UserAgent',
        sub {
            my ($VerifyHostname) = @_;
            my $CustomUserAgent = $self->_getMockedUserAgent();
            $CustomUserAgent->ssl_opts( verify_hostname => $VerifyHostname ) unless $VerifyHostname;
            return $CustomUserAgent;
        }
    );

    return $MockedCompiledClient;
}

sub _getMockedUserAgent {
    my $self = shift;
    my ($Response) = @_;

    my $UserAgent       = LWP::UserAgent->new();
    my $MockedUserAgent = Test::MockObject::Extends->new($UserAgent);
    $MockedUserAgent->mock(
        'request',
        sub {
            my $self = shift;
            my ($Request) = @_;
            $self->{'Request_MockVar'} = $Request->{'_content'};
            return $Response;
        }
    );

    return $MockedUserAgent;
}

__PACKAGE__->runScript();
