#========================================================================================
# package      DE_EPAGES::Ebay::API::Object::LineItemEbayTransaction
# base         DE_EPAGES::Order::API::Object::LineItemProduct
# state        public
#----------------------------------------------------------------------------------------
# description  object interface for LineItemEbayTransaction.
#========================================================================================
package DE_EPAGES::Ebay::API::Object::LineItemEbayTransaction;
use base DE_EPAGES::Product::API::Object::LineItemProduct;

use strict;

sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);

#    TriggerHook('OBJ_InsertLineItemProduct', {'Object' => $self, 'LineItemID' => $self->id });
    return $self;
}

#========================================================================================
# function     recalculate
#----------------------------------------------------------------------------------------
# syntax       $LineItem->recalculate($hValues);
#----------------------------------------------------------------------------------------
# description
#----------------------------------------------------------------------------------------
# input        $hValues | values witch will be changed | ref.hash
#========================================================================================
sub recalculate {
    my $self = shift;
    my ($Container, $hValues) = @_;
    my $Transaction = $self->get('EbayTransaction');

    return $self->SUPER::recalculate($Container, $hValues) unless defined $Transaction;

    my $Quantity = $Transaction->get('Quantity');
    $Quantity ||= 1;

    #to be able to edit CreditNotes, Packingslips and Invoices for Amazon orders we need to determine if the Baseclass
    #for the container is of typ OrderDocument
    #if $hValues hash is empty we use the values from the original Transaction
    my $LineItemContainerParentObject = $self->parent->parent;
    if (keys(%$hValues) and $LineItemContainerParentObject->instanceOf('OrderDocument')){
        $self->set({
            'Quantity'      => $hValues->{'LineItems'}[0]->{'Quantity'} // $self->get('Quantity'),
            'BasePrice'     => $hValues->{'LineItems'}[0]->{'BasePrice'} // $self->get('BasePrice'),
            'LineItemPrice' => $Transaction->get('Price') * $hValues->{'LineItems'}[0]->{'Quantity'} // $Transaction->get('Price') * $Transaction->get('Quantity'),
        });
    } else {
        $self->set({
            'Quantity'      => $Transaction->get('Quantity'),
            'BasePrice'     => $Transaction->get('Price'),
            'LineItemPrice' => $Transaction->get('Price') * $Quantity,
        });
    }
    #Call recalculate to calc Prices etc.
    return $self->SUPER::recalculate($Container, $hValues);
}

#========================================================================================
# function     isQuantityCorrectOnInsertOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemEbayTransaction->isQuantityCorrectOnInsertOrder;
#----------------------------------------------------------------------------------------
# description  No quantity check for eBay transactions, product may have no price in
#               MBO which is okay since price comes from eBay.
#----------------------------------------------------------------------------------------
# return       $IsValid | true if the quantity is ok | boolean
#========================================================================================
sub isQuantityCorrectOnInsertOrder {
    my $self = shift;

    return 1;
}

1;
