#========================================================================================
# §package      DE_EPAGES::PayPalPPCP::API::Object::LineItemPaymentPayPalPPCP
# §state        public
# §base         DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal
#----------------------------------------------------------------------------------------
# §description  object interface for LineItemPaymentPayPalPPCP.
#========================================================================================
package DE_EPAGES::PayPalPPCP::API::Object::LineItemPaymentPayPalPPCP;
use base DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal;

use DateTime::Format::Strptime;

use DE_EPAGES::PayPalPPCP::API::Constants qw ( $ORDER_STATUS_COMPLETED );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::XML::API::DateTimeFormatXML;

use strict;

#========================================================================================
# §function     className
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $ClassName = $LineItemPaymentPayPalPPCP->className;
#----------------------------------------------------------------------------------------
# §description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# §return       $ClassName | class name | String
#========================================================================================
sub className {
    return 'LineItemPaymentPayPalPPCP';
}

#========================================================================================
# §function     insert
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $LineItemPaymentPayPalPPCP = DE_EPAGES::PayPalPPCP::API::Object::LineItemPaymentPayPalPPCP->insert($hInfo);
# §example      my $Class = LoadClassByAlias('LineItemPaymentPayPalPPCP');
#               my $LineItemPaymentPayPalPPCP = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# §description  Insert this info as object, afterwards the hook 'OBJ_InsertLineItemPaymentPayPalPPCP' is triggered.
#----------------------------------------------------------------------------------------
# §input        $Package | class package | string
# §input        $hInfo | attributes of new object | ref.hash
# §return       $LineItemPaymentPayPalPPCP | new object | object
# §hook         OBJ_InsertLineItemPaymentPayPalPPCP | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentPayPalPPCPID - line item payment id - int
#               </ul>
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertLineItemPaymentPayPalPPCP', {
        'Object' => $self,
        'LineItemPaymentPayPalPPCPID' => $self->id()
    });

    return $self;
}

#========================================================================================
# §function     delete
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $LineItemPaymentPayPalPPCP->delete;
#----------------------------------------------------------------------------------------
# §description  Deletes this object, therefore the hook 'OBJ_DeleteLineItemPaymentPayPalPPCP' is triggered.
#----------------------------------------------------------------------------------------
# §hook         OBJ_DeleteLineItemPaymentPayPalPPCP | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentPayPalPPCPID - line item payment id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteLineItemPaymentPayPalPPCP', {
        'Object' => $self,
        'LineItemPaymentPayPalPPCPID' => $self->id()
    });

    $self->SUPER::delete();
    return;
}

#========================================================================================
# §function     set
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $LineItemPaymentPayPalPPCP->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# §description  This function triggers hooks 'OBJ_BeforeUpdateLineItemPaymentPayPalPPCP'
#               and 'OBJ_AfterUpdateLineItemPaymentPayPalPPCP'.
#----------------------------------------------------------------------------------------
# §input        $hValues | attribute name => value hash | ref.hash
# §input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# §hook         OBJ_BeforeUpdateLineItemPaymentPayPalPPCP | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentPayPalPPCPID - line item payment id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# §hook         OBJ_AfterUpdateLineItemPaymentPayPalPPCP | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentPayPalPPCPID - line item payment id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateLineItemPaymentPayPalPPCP', {
        'Object' => $self,
        'Values' => $hValues,
        'LineItemPaymentPayPalPPCPID' => $self->id(),
        'LanguageID' => $LanguageID
    });

    $self->SUPER::set($hValues, $LanguageID);

    TriggerHook('OBJ_AfterUpdateLineItemPaymentPayPalPPCP', {
        'Object' => $self,
        'Values' => $hValues,
        'LineItemPaymentPayPalPPCPID' => $self->id(),
        'LanguageID' => $LanguageID
    });

    return;
}

#========================================================================================
# §function     updateLineItemFromPPCP
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $LineItemPaymentPayPalPPCP->updateLineItemFromPPCP();
#----------------------------------------------------------------------------------------
# §description  Based on the order information from PayPal, update the line item information
#----------------------------------------------------------------------------------------
#========================================================================================
sub updateLineItemFromPPCP {
    my $self = shift;

    my $hOrderInfo = $self->get('OrderInfoJSON');
    return unless $hOrderInfo;

    return unless $hOrderInfo->{'status'} eq $ORDER_STATUS_COMPLETED;
    my $hPurchaseUnit = $hOrderInfo->{'purchase_units'}->[0];
    return unless $hPurchaseUnit;

    my $Amount = $hPurchaseUnit->{'amount'}->{'value'};
    my $TransID = $hPurchaseUnit->{'payments'}->{'captures'}->[0]->{'id'};

    #-- The 'payer' object is deprecated, but still returned, so use it as a backup
    my $UserID = exists $hOrderInfo->{'payment_source'}->{'paypal'}->{'account_id'} ? $hOrderInfo->{'payment_source'}->{'paypal'}->{'account_id'} : 
                 exists $hOrderInfo->{'payer'}->{'payer_id'} ? $hOrderInfo->{'payer'}->{'payer_id'} : undef;

    my $DtParser = DateTime::Format::Strptime->new( pattern => '%Y-%m-%dT%H:%M:%SZ' );
    my $TransTime;
    eval {
        $TransTime = $DtParser->parse_datetime($hOrderInfo->{'create_time'});
    };
    $TransTime = undef if $@;

    $self->set({
        #'CurrencyID' => ,
        'Amount' => $Amount,
        'TransID' => $TransID,
        'TransTime' => defined $TransTime ? $TransTime : undef,
        'UserID' => $UserID,
    });
}

1;