#========================================================================================
# nocomments
# package      DE_EPAGES::ServiRed::API::Constants
# state        private
# description  constants for cartridge
#========================================================================================
package DE_EPAGES::ServiRed::API::Constants;
use base qw (Exporter);

use strict;
our @EXPORT_OK = qw (NO_SERVIRED_LANGUAGE
                  %SERVIRED_LANGUAGES
                  %SERVIRED_ERRORCODES
                  %SERVIRED_UNENCRYPTED_PARAMETERS
                  STATUS_AUTHORISED
                  STATUS_CANCELLED
                  STATUS_CAPTURED
                  STATUS_DECLINED
                  STATUS_NEW
                  DS_MERCHANT_PAYMETHODS_BIZUM
                  DS_MERCHANT_PAYMETHODS_SERVIRED);

#--- transaction status
use constant STATUS_NEW        => 'N';
use constant STATUS_AUTHORISED => 'A';
use constant STATUS_CANCELLED  => 'X';
use constant STATUS_CAPTURED   => 'C';
use constant STATUS_DECLINED   => 'D';

#--- ServiRed languages
our %SERVIRED_LANGUAGES = (
    'es' => '001', # Spanish
    'en' => '002', # English
    'ca' => '003', # Catalan
    'fr' => '004', # French
    'de' => '005', # German
    'nl' => '006', # Dutch
    'it' => '007', # Italian
    'sv' => '008', # Swedish
    'pt' => '009', # Portuguese
    # Valencian (010) omitted
);

#========================================================================================
#   Return Codes from ServiRed
#   OK:
#    0000  0099  Transaccin autorizada para pagos y preautorizaciones
#                > Authorizierte Transaktionen und pre-authorizierung
#    0900  Transaccin autorizada para devoluciones y confirmaciones >
#   Error:
#    101 Tarjeta caducada > Karte mit ungltige Gltichkeitsdatum
#    102  Tarjeta en excepcin transitoria o bajo sospecha de fraude  > Karte unter Betrug Verdacht oder Derogation
#    104  Operacin no permitida para esa tarjeta o terminal > Operation nicht erlaubt fr diese Karte oder Terminal
#    116 Disponible insuficiente > Dispo Ungengend
#
#    118  Tarjeta no registrada > Karte nicht regristriert
#    129  Cdigo de seguridad (CVV2/CVC2) incorrecto > Sicherheitscode ungltig
#    180  Tarjeta ajena al servicio > Karte gehrt nicht zum Service
#    184  Error en la autenticacin del titular > Fehler bei der Authentifizierung der Karteinhaber
#    190  Denegacin sin especificar Motivo  > Ablehnung ohne specifische Grnde
#    191  Fecha de caducidad errnea > Falsches Gltigkeitsdatum
#    202 Tarjeta en excepcin transitoria o bajo sospecha de fraude con retirada de tarjeta
#         > Karte unter Betrug Verdacht oder Derogation mit Kartesperre (oder sie nehmen die Karte einfach weg).
#    912/9912  Emisor no disponible ualquier otro valor  Transaccin denegada
#    >  Emittenten nicht verfgbar alle andere Transaktionen Verweigert
#
#    https://pagosonline.redsys.es/codigosRespuesta.html
#========================================================================================
our %SERVIRED_ERRORCODES = (
    '0000' => 'TRANSACTION_SUCCESSFUL',
    '0099' => 'TRANSACTION_SUCCESSFUL',
    '0900' => 'TRANSACTION_SUCCESSFUL',
    '0913' => 'TRANSACTION_REPEATED',
    '9913' => 'TRANSACTION_REPEATED',
    '0915' => 'TRANSACTION_CANCELLED',
    '9915' => 'TRANSACTION_CANCELLED',
    '9673' => 'TRANSACTION_CANCELLED',
    '0129' => 'AUTHENTICATION_ERROR',
    '0184' => 'AUTHENTICATION_ERROR',
    '0101' => 'CARD_EXPIRED_OR_SUSPENDED',
    '0102' => 'CARD_EXPIRED_OR_SUSPENDED',
    '0191' => 'CARD_EXPIRED_OR_SUSPENDED',
    '0202' => 'CARD_EXPIRED_OR_SUSPENDED',
    '0104' => 'CARD_UNKNOWN',
    '0118' => 'CARD_UNKNOWN',
    '0180' => 'CARD_UNKNOWN',
);

use constant NO_SERVIRED_LANGUAGE => '0';

our %SERVIRED_UNENCRYPTED_PARAMETERS = (
    'DS_MERCHANT_AMOUNT' => 'merchantAmount',
    'DS_MERCHANT_ORDER'=> 'merchantOrder',
    'DS_MERCHANT_MERCHANTCODE' => 'merchantCode',
    'DS_MERCHANT_CURRENCY' => 'merchantCurrency',
    'DS_MERCHANT_TRANSACTIONTYPE' => 'merchantTransactionType',
    'DS_MERCHANT_TERMINAL' => 'merchantTerminal',
    'DS_MERCHANT_CONSUMERLANGUAGE'=> 'merchantConsumerLanguage',
    'DS_MERCHANT_MERCHANTURL' => 'merchantUrl',
    'DS_MERCHANT_URLOK' => 'merchantUrlOk',
    'DS_MERCHANT_URLKO' => 'merchantUrlKo',
    'DS_MERCHANT_PRODUCTDESCRIPTION' => 'merchantProductDescription',
    'DS_MERCHANT_PAYMETHODS' => 'merchantPayMethods',
    'secretKey' => 'secretKey',
);

#--- Param 'PayMethods' values
use constant DS_MERCHANT_PAYMETHODS_BIZUM => 'z';
use constant DS_MERCHANT_PAYMETHODS_SERVIRED => 'C';

1;
