#========================================================================================
# package      DE_EPAGES::TrustedShops::t::unit::TrustedShopsCertificateHandler
# base         DE_EPAGES::Core::API::Test
# state        public
# description  Unit test for TrustedShopsCertificateHandler
#========================================================================================
package DE_EPAGES::TrustedShops::t::unit::TrustedShopsCertificateHandler;
use base DE_EPAGES::Core::API::Test;

use strict;

use Test::More;
use Test::MockObject::Extends;
use Test::Exception;
use Test::Mockify;
use Test::Mockify::Matcher qw ( Number String Object);
use Test::Mockify::Verify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;

use DE_EPAGES::TrustedShops::API::Constants qw ( %CERT_STATUS %IS_EXCELLENCE_CERTIFICATE );

#----------------------------------------------------------------------------------------
sub test {
    my $self = shift;
    $self->_test_excellence_certificate();
    $self->_test_classic_certificate();
    $self->_test_excellence_certificate_Error_INVALID_CERTIFICATE_no_certification_language();
    $self->_test_excellence_certificate_Error_INVALID_CERTIFICATE_unexpected_status();
    $self->_test_excellence_certificate_Error_NON_EXCELLENCE_CERTIFICATION();
    $self->_test_excellence_certificate_Error_LANGUAGE_CERTIFICATE_EXISTS();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub _test_excellence_certificate {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsExcellence = 1;
    my $LocalId = 'de';
    my $TrustedShopsCertificateHandler = $self->_createSUT( $IsExcellence, $LocalId, 'X22222222222222222222222222222222');

    is( $TrustedShopsCertificateHandler->createCertificateObject(), 12345 , "$SubTestName - tests classic certificate creation");
    
    my $aParams =  GetParametersFromMockifyCall( $TrustedShopsCertificateHandler, '_createTrustedShopsCertificate' );
    isa_ok($aParams->[0], 'DE_EPAGES::Object::API::Object::Object', "$SubTestName - proves that the first parameter is a Object");
    is($aParams->[0]->alias(), 'FolderObject', "$SubTestName - proves that the first parameter is an expected 'FolderObject'");
    is($aParams->[1], 'X22222222222222222222222222222222', "$SubTestName - proves that the second parameter is the expected 'DummyTrustedShopId'");
    is($aParams->[2], $LocalId, "$SubTestName - proves that the third parameter is the expected 'LocalId'");
    is($aParams->[3], $CERT_STATUS{'TEST'}, "$SubTestName - proves that the fourth parameter is the expected cert Status: 'TEST'");
    is($aParams->[4], $IS_EXCELLENCE_CERTIFICATE{'YES'}, "$SubTestName - proves that the fifth parameter is the expected 'excellence certificate'");
}
#----------------------------------------------------------------------------------------
sub _test_classic_certificate {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsExcellence = 0;
    my $LocalId = 'de';
    my $TrustedShopsCertificateHandler = $self->_createSUT( $IsExcellence, $LocalId, 'DummyTrustedShopId');

    is( $TrustedShopsCertificateHandler->createCertificateObject(), 12345 , "$SubTestName - tests classic certificate creation");

    my $aParams =  GetParametersFromMockifyCall( $TrustedShopsCertificateHandler, '_createTrustedShopsCertificate' );
    isa_ok($aParams->[0], 'DE_EPAGES::Object::API::Object::Object', "$SubTestName - proves that the first parameter is a Object");
    is($aParams->[0]->alias(), 'FolderObject', "$SubTestName - proves that the first parameter is an expected 'FolderObject'");
    is($aParams->[1], 'DummyTrustedShopId', "$SubTestName - proves that the second parameter is the expected 'DummyTrustedShopId'");
    is($aParams->[2], $LocalId, "$SubTestName - proves that the third parameter is the expected 'LocalId'");
    is($aParams->[3], $CERT_STATUS{'OTHER'}, "$SubTestName - proves that the fourth parameter is the expected 'cert Status'");
    is($aParams->[4], $IS_EXCELLENCE_CERTIFICATE{'NO'}, "$SubTestName - proves that the fifth parameter is the expected 'non excellence certificate'");
}
#----------------------------------------------------------------------------------------
sub _test_excellence_certificate_Error_INVALID_CERTIFICATE_no_certification_language {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsExcellence = 1;
    my $LocalId = 'de';
    my $TrustedShopsCertificateHandler = $self->_createSUT( $IsExcellence, $LocalId, 'X11111111111111111111111111111111');

    throws_ok( sub { $TrustedShopsCertificateHandler->createCertificateObject() },
                   qr/INVALID_CERTIFICATE/,
                   "$SubTestName - prove die if certificated is invalid ( no languange )"
         );
}
#----------------------------------------------------------------------------------------
sub _test_excellence_certificate_Error_INVALID_CERTIFICATE_unexpected_status {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsExcellence = 1;
    my $LocalId = 'de';
    my $TrustedShopsCertificateHandler = $self->_createSUT( $IsExcellence, $LocalId, 'X55555555555555555555555555555555');

    throws_ok( sub { $TrustedShopsCertificateHandler->createCertificateObject() },
                   qr/INVALID_CERTIFICATE/,
                   "$SubTestName - prove die if certificated is invalid ( unknown status )"
         );
}
#----------------------------------------------------------------------------------------
sub _test_excellence_certificate_Error_NON_EXCELLENCE_CERTIFICATION {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsExcellence = 1;
    my $LocalId = 'de';
    my $TrustedShopsCertificateHandler = $self->_createSUT( $IsExcellence, $LocalId, 'X33333333333333333333333333333333');

    throws_ok( sub { $TrustedShopsCertificateHandler->createCertificateObject() },
                   qr/NON_EXCELLENCE_CERTIFICATION/,
                   "$SubTestName - prove die if certificated is invalid ( wrong certificate type )"
         );
}
#----------------------------------------------------------------------------------------
sub _test_excellence_certificate_Error_LANGUAGE_CERTIFICATE_EXISTS {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsExcellence = 1;
    my $LocalId = 'de';
    my $TrustedShopsCertificateHandler = $self->_createSUT( $IsExcellence, $LocalId, 'X44444444444444444444444444444444');

    throws_ok( sub { $TrustedShopsCertificateHandler->createCertificateObject() },
                   qr/LANGUAGE_CERTIFICATE_EXISTS/,
                   "$SubTestName - prove die if certificated is invalid ( languange is already set )"
         );
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($IsExcellence, $CertificateLanguage, $TrustedShopsID) = @_;
    my $Mockify = Test::Mockify->new(
        'DE_EPAGES::TrustedShops::API::TrustedShopsCertificateHandler',
        [
            $self->_createShop(),
            $TrustedShopsID,
            $IsExcellence,
            $CertificateLanguage,
        ]
    );
    $Mockify->mock('_GetInfoLocale')
        ->when(String('en_GB'))
        ->thenReturn({ 'LanguageID' => 2 });

    $Mockify->mock('_GetInfoLanguage')
        ->when(Number(2))
        ->thenReturn({'Code2' => 'en'});

    $Mockify->mock('_LoadClassByAlias')
        ->when(String('TrustedShopsCertificate'))
        ->thenReturn($self->_createTrustedShopCertificateClass());

    $Mockify->mock('_CheckCertificate')
        ->when(String('X11111111111111111111111111111111'), Object())
        ->thenReturn({
              'tsID' => 'X11111111111111111111111111111111',
              'url' => 'www.trustedshops.de',
              'typeEnum' => 'EXCELLENCE',
              'stateEnum' => 'TEST'
            });

    $Mockify->mock('_CheckCertificate')
        ->when(String('X22222222222222222222222222222222'), Object())
        ->thenReturn({
              'certificationLanguage' => 'de',
              'tsID' => 'X22222222222222222222222222222222',
              'url' => 'www.trustedshops.de',
              'typeEnum' => 'EXCELLENCE',
              'stateEnum' => 'TEST'
            });

    $Mockify->mock('_CheckCertificate')
        ->when(String('X33333333333333333333333333333333'), Object())
        ->thenReturn({
              'certificationLanguage' => 'de',
              'tsID' => 'X33333333333333333333333333333333',
              'url' => 'www.trustedshops.de',
              'typeEnum' => 'NoEXCELLENCE',
              'stateEnum' => 'TEST'
            });

    $Mockify->mock('_CheckCertificate')
        ->when(String('X44444444444444444444444444444444'), Object())
        ->thenReturn({
              'certificationLanguage' => 'ru',
              'tsID' => 'X44444444444444444444444444444444',
              'url' => 'www.trustedshops.de',
              'typeEnum' => 'EXCELLENCE',
              'stateEnum' => 'TEST'
            });

    $Mockify->mock('_CheckCertificate')
        ->when(String('X55555555555555555555555555555555'), Object())
        ->thenReturn({
        'certificationLanguage' => 'de',
        'tsID' => 'X55555555555555555555555555555555',
        'url' => 'www.trustedshops.de',
        'typeEnum' => 'EXCELLENCE',
        'stateEnum' => 'NO_AUDIT'
    });

    $Mockify->spy('_createTrustedShopsCertificate')->whenAny();

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'LocaleID' => 'en_GB',
        'TrustedShopsCertificates'  => [
            $self->_createTrustedShopCertificate(),
        ],
        'IsInternalTestShop' => 1,
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop',
            $hAttributes
    );

    $Mockify->mock('folder', $self->_createFolder(), [{'string' => 'TrustedShopsCertificates'}]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createFolder {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'FolderObject'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Object',
        $hAttributes
    );

    $Mockify->mock(
        'insertChild',
        $self->_createTrustedShopCertificate(),
        [
            {'object' => 'DE_EPAGES::Object::API::Object::Class'},
            'hashref'
        ]
    );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createTrustedShopCertificateClass {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'MyShop'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Class',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createTrustedShopCertificate {
    my $self = shift;

    my $hAttributes = {
        'ID' => 12345,
        'CertificationLanguage' => 'ru'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::TrustedShops::API::Object::TrustedShopsCertificate',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
__PACKAGE__->runScript();