# CONTENTS

This document lists example error responses from Zettle API. Note that API response is a JSON encoded string.

List is indexed and ordered alphabetically by ErrorType.

## ACCESS_TOKEN_EXPIRED
You get this error when Zettle received an invalid or expired access token. We now use the refresh token to acquire a new access token before it expires.<br/>
Refer to [_sendRequestWithRefreshWrapper()](https://github.com/ePages-de/Cartridges/blob/develop/DE_EPAGES/Zettle/API/WebService/RestService.pm#L287) method for implementation details.<br/>
See also section [Generate a new access token from the refresh token](https://developer.zettle.com/docs/api/oauth/api-reference#generate-a-new-access-token-from-the-refresh-token) from Zettle documentation.
```
{"developerMessage":"Access token expired at: NumericDate{1652107028 -> May 9, 2022, 2:37:08 PM UTC}","errorType":"ACCESS_TOKEN_EXPIRED"}
```

## CONSTRAINT_VIOLATION

### CONSTRAINT_VIOLATION: destination must be valid https URL
Zettle doesn't allow plain HTTP resources. All resources must use secure HTTPS.
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations": [{"propertyName":"validDestination","developerMessage":"Destination must be valid https URL","constraintType":"UNKNOWN","value":"false"}, {"propertyName":"destination","developerMessage":"size must be between 1 and 5...
```

### CONSTRAINT_VIOLATION: invalid variant size. Max: 300 (variants)
Zettle imposes a limit on the number of variants (subproducts) per product. Even though Zettle API response states "Max: 300 (variants)", official documentation states this limit is 90 variants per product.<br/>
See [Product library data relationships](https://developer.zettle.com/docs/api/product-library/concepts/product-library-data#relationships) section.
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"propertyName":"variants","developerMessage":"Invalid variant size. Max: 300","constraintType":"UNKNOWN","value":"FullProductUpdateRequest(uuid=635c8cbb-5522-104c-25e7-d580921e1114, categories=[], name=UD-107 Test-01, description=Optional.empty, imageLookupKeys=[], presentation=Opti...
```
This error was generated by an export request which included a product with 343 variants (the combination of three variation attributes having 7 values each leading to 7x7x7=343).

### CONSTRAINT_VIOLATION: list of variants must not be empty (variants)
Unlike ePages, Zettle need all products to have at least one variant. [Product library data](https://developer.zettle.com/docs/api/product-library/concepts/product-library-data#example-of-product-and-variants) documentation states `the variants array must contain at least one entry` and provide a request body example.<br/>

```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION", "violations":[{"propertyName":"variants","developerMessage":"List of variants must not be empty","constraintType":"UNKNOWN","value": "FullProductUpdateRequest(uuid=60f3f266-4932-1a1e-2019-b976395fee89, categories...
```

### CONSTRAINT_VIOLATION: multivariant product needs to contain variantOptionDefinitions
As stated in [Products and variants](https://developer.zettle.com/docs/api/product-library/concepts/product-library-data#products-and-variants) documentation `variantOptionDefinitions include the name of the variant with all the values for that variant`. This means that `variantOptionDefinitions` field must be present and must match the combination of all `variants.options` attributes (name/value).
See [Example of product and variants](https://developer.zettle.com/docs/api/product-library/concepts/product-library-data#example-of-product-and-variants) for a sample request.
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"developerMessage":"Multivariant product needs to contain variantOptionDefinitions","constraintType":"UNKNOWN","value":"FullProductUpdateRequest(uuid=6397a525-4bd6-1897-292e-d580921e33a1, categories=[], name=UD-107 Test Price Export >99, description=Optional.empty, imageLookupKeys=[], presentation=Optional.empty, variants=[VariantDTO(uuid=6397a525-a010-10fa-2100-d580921e33b3, name=Optional.empty, description=Optional.empty, sku=Optional.empty, barcode=Optional.empty, price=Optional[Price(currencyId=EUR, amount=35400)], costPrice=Optional.empty, options=[VariantOption(name=Attribute 5 options A, value=5A-three), VariantOption(name=Attribute 5 options B, value=5B-five), VariantOption(name=Attribute 4 options, value=4-four)], presentation=Optional.empty), VariantDTO(uuid=6397a525-74c4-1d30-21bb-d580921e33b3, name=Optional.empty, description=Optional.empty, sku=Optional.empty, barcode=Optional.empty, price=Optional.empty, costPrice=Optional.empty, options=null, presentation=Optional.empty)], externalReference=Optional.empty, unitName=Optional.empty, vatPercentage=Optional[19], online=Optional.empty, variantOptionDefinitions=Optional.empty, taxCode=null, category=Optional.empty, metadata=Optional[MetadataDTO(inPos=true, source=Optional[SourceDTO(name=epages, external=true)])], taxRates=null, catalogAssignments=Optional.empty, taxExempt=null)"}]}
```
We see that variants.options lists 3 attributes but `variantOptionDefinitions` is empty.

### CONSTRAINT_VIOLATION: must be greater than 0 (productchanges.variantChanges.change)
Zettle complains when we send a stock update request with a change of 0 units.
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"propertyName":"productChanges[3].variantChanges[].change","developerMessage":"must be greater than 0","constraintType":"MUST_BE_GREATER_THAN","expected":"0","value":"0"},{"propertyName":"productChanges[0].variantChanges[].change","developerMessage":"must be greater than 0","constraintType":"MUST_BE_GREATER_THAN","expected":"0","value":"0"}]}
```

Error was generated by the following request.
```
{"productChanges":[{"trackingStatusChange":"START_TRACKING","variantChanges":[{"change":0,"fromLocationUuid":"632be6fe-f093-11eb-bca6-6fb0a1c64855","toLocationUuid":"632be7d4-f093-11eb-b143-ca33b7c92053","variantUuid":"628f7868-161f-1f8f-28d9-d580921e9262"}],"productUuid":"628f7868-161f-1f8f-28d9-d580921e9262"},{"trackingStatusChange":"START_TRACKING","variantChanges":[{"change":20,"fromLocationUuid":"632be6fe-f093-11eb-bca6-6fb0a1c64855","toLocationUuid":"632be7d4-f093-11eb-b143-ca33b7c92053","variantUuid":"628f7868-b2eb-1653-2db1-d580921e925f"}],"productUuid":"628f7868-b2eb-1653-2db1-d580921e925f"},...
```
Notice `change` field with a value of 0.

### CONSTRAINT_VIOLATION: must not be blank (variants.options.value)
Variant options cannot be empty. `options` field must contain non-empty `name` and `value` fields.<br/>
See [Example of products and variants](https://developer.zettle.com/docs/api/product-library/concepts/product-library-data#example-of-product-and-variants).
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"propertyName":"variants[1].options[1].value","developerMessage":"must not be blank","constraintType":"UNKNOWN"},{"propertyName":"variants[3].options[1].value","developerMessage":"must not be blank","constraintTyp...
```

### CONSTRAINT_VIOLATION: must not be null (variants.price.currencyId)
`currencyId` field must contain a valid currency code. CurrencyId value must match one of the enums listed for request.variants.price.currencyId in [updateProduct](https://developer.zettle.com/docs/api/product-library/reference#updateProduct) request.<br/>
See [Example of products and variants](https://developer.zettle.com/docs/api/product-library/concepts/product-library-data#example-of-product-and-variants).
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations": [{"propertyName":"variants[1].price.currencyId","developerMessage":"must not be null","constraintType":"MUST_NOT_BE_NULL"},{"propertyName": "variants[0].price.currencyId","developerMessage":"must not be null","constr...
```

### CONSTRAINT_VIOLATION: one or more item in collection was not unique (variants)
This is a sign of duplicated UUIDs. Error occurs when the same UUID is shared by several variants for the same superproduct.
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations": [{"propertyName":"variants","developerMessage":"One or more item in collection was not unique: [6156c55e-6f13-1466-2b51-b976395f55be, 6156c55e-f307-152e-28af-b976395f55a8, 6156c55e-6f13-1466-2b51-b976395f55be, 6156c...
```

### CONSTRAINT_VIOLATION: options not valid. Either missing or undefined values
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"developerMessage":"Options not valid. Either missing or undefined values: [VariantOption(name=Colour, value=blue), VariantOption(name=Material,value=Cotton), VariantOption(name=Colour, value=blue), VariantOption...
```

### CONSTRAINT_VIOLATION: UUID is required and must be version 1 (changes.toLocationUuid)
Identifiers generated need to conform with UUIDv1 (refer to [RFC4122](https://www.ietf.org/rfc/rfc4122.txt)).
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"propertyName":"changes[].toLocationUuid","developerMessage":"{UUID is required and must be version 1}","constraintType":"UNKNOWN"}]}
```

### CONSTRAINT_VIOLATION: UUID is required and must be version 1 (productchanges.productUuid)
Identifiers generated need to conform with UUIDv1 (refer to [RFC4122](https://www.ietf.org/rfc/rfc4122.txt)).
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"propertyName":"productChanges[1].productUuid","developerMessage":"{UUID is required and must be version 1}","constraintType":"UNKNOWN"},{"propertyName":"productChanges[1].productUuid","developerMessage":"must not be null","constraintType":"MUST_NOT_BE_NULL"}]}
```

### CONSTRAINT_VIOLATION: UUID is required and must be version 1 (productChanges.variantChanges.variantUuid)
Identifiers generated need to conform with UUIDv1 (refer to [RFC4122](https://www.ietf.org/rfc/rfc4122.txt)).
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"propertyName":"productChanges[0].variantChanges[].variantUuid","developerMessage":"{UUID is required and must be version 1}","constraintType":"UNKNOWN"}]}
```

### CONSTRAINT_VIOLATION: UUID is required and must be version 1 (returnBalanceForLocationUuid)
Identifiers generated need to conform with UUIDv1 (refer to [RFC4122](https://www.ietf.org/rfc/rfc4122.txt)).
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"propertyName":"returnBalanceForLocationUuid","developerMessage":"{UUID is required and must be version 1}","constraintType":"UNKNOWN"}]}
```

### CONSTRAINT_VIOLATION: variants expected to have X variant options present
Attributes for variant options must match the options specified in `variantOptionDefinitions`.<br/>
See [Example of products and variants](https://developer.zettle.com/docs/api/product-library/concepts/product-library-data#example-of-product-and-variants).
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"developerMessage":"Variants expected to have 2 variant options present","constraintType":"UNKNOWN","value":"FullProductUpdateRequest(uuid=623483e5-4fbb-1e6f-2478-d580921eb441, categories=[], name=UD-39-TestProduct01, description=Optional.empty, imageLookupKeys=[iCwM_oJiEeug_vfmbRm7CQ-fbky17DmEey8KZoUyqqDJg], presentation=Optional.empty, variants=[VariantDTO(uuid=623483e5-ff12-160a-2320-d580921eb4cb, name=Optional[UD-39-TestProduct01], description=Optional.empty, sku=Optional.empty, barcode=Optional.empty, price=Optional.empty, costPrice=Optional.empty, options=[VariantOption(name=Size, value=M)], presentation=Optional.empty), VariantDTO(uuid=623483e5-4e34-153a-271e-d580921eb40e, name=Optional[UD-39-TestProduct01], description=Optional.empty, sku=Optional.empty, barcode=Optional.empty, price=Optional.empty, costPrice=Optional.empty, options=[VariantOption(name=Size, value=M)], presentation=Optional.empty), VariantDTO(uuid=623483e5-a8ca-1a8c-24b6-d580921eb4fe, name=Optional[UD-39-TestProduct01], description=Optional.empty, sku=Optional.empty, barcode=Optional.empty, price=Optional.empty, costPrice=Optional.empty, options=[VariantOption(name=Size, value=L)], presentation=Optional.empty), VariantDTO(uuid=623483e5-c389-1bb5-2d2f-d580921eb4f4, name=Optional[UD-39-TestProduct01], description=Optional.empty, sku=Optional.empty, barcode=Optional.empty, price=Optional.empty, costPrice=Optional.empty, options=[VariantOption(name=Size, value=L)], presentation=Optional.empty)], externalReference=Optional[62347593-A71C-BD6F-B6C9-D580921EB76D], unitName=Optional.empty, vatPercentage=Optional[21], online=Optional.empty, variantOptionDefinitions=Optional[VariantOptionDefinitions(definitions=[VariantOptionDefinition(name=Color, properties=[VariantOptionProperties(value=Red, imageUrl=Optional.empty), VariantOptionProperties(value=Blue C, imageUrl=Optional.empty)]), VariantOptionDefinition(name=Size, properties=[VariantOptionProperties(value=M, imageUrl=Optional.empty), VariantOptionProperties(value=L, imageUrl=Optional.empty)])])], taxCode=null, category=Optional.empty, metadata=Optional[MetadataDTO(inPos=true, source=Optional[SourceDTO(name=epages, external=true)])], taxRates=null, taxExempt=null)"}]}
```
We can see that `variantOptionDefinitions` list two attributes, `Color` and `Size`, while variants.options only include `Size` attribute, missing `Color` attribute.

In the following sample we see the same error but with a different number of options. `variantOptionDefinitions` field might list 3 options and some of the variant options didn't match that number of options.
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"developerMessage":"Variants expected to have 3 variant options present","constraintType":"UNKNOWN","value":"FullProductUpdateRequest(uuid=63971776-469d-1109-2760-d580921e359e, categories=[], name=UD-107 Test 100 variants SUCCESS ?, description=Optional[<p>Tes...
```

## DESTINATION_RESPONDED_WITH_ERROR_CODE
Callback URL must be reachable by Zettle.
```
{"developerMessage":"Could not create webhook, receiving webhook endoint responded with http 404","errorType":"DESTINATION_RESPONDED_WITH_ERROR_CODE","violations":[]}
```

```
{"developerMessage":"Could not create webhook, receiving webhook endoint responded with http 500","errorType": "DESTINATION_RESPONDED_WITH_ERROR_CODE","violations":[]}
```

## ENTITY_NOT_FOUND
Error response when requesting a product that is not in Zettle.
```
{"developerMessage":"PRODUCT_V2 with UUID 6384b5f7-c110-1350-2892-d580921ed0ca not found","errorType":"ENTITY_NOT_FOUND","violations":[]}
```

## NO_ITEMS_TO_IMPORT
```
{"developerMessage":"Expected property \'products\' with at least one item for organization 1a52e2a8-824d-11eb-acc8-aafa2648b59f","errorType":"NO_ITEMS_TO_IMPORT","violations":[]}
```

## PRODUCT_NOT_TRACKED
Zettle uses the concept of tracking to refer to products for which stock level changes will be monitored and, if setup, generate callbacks to the shop.<br/>
See [How inventories work](https://developer.zettle.com/docs/api/inventory-v3/concepts/how-inventories-work) from Zettle documentation.
```
{"developerMessage":"Product not tracked. UUID=635c8cc3-59c0-1c59-2063-d580921e1121","errorType":"PRODUCT_NOT_TRACKED","violations":[]}
```

## IMPORT_ALREADY_STARTED
This error is caused when sending several requests to Zettle in a short timespan.<br/>
We introduced a delay between product export requests to avoid this error.
```
{"developerMessage":"Import already running with UUID d712e89c-ad30-11ed-93f4-7fc7346ed7bc","errorType":"IMPORT_ALREADY_STARTED","violations":[]}
```

## IMPORT_LIMIT_REACHED
Zettle limits the maximum number of items to be imported within a period of 24 hours.<br/>
See [Bulk import of products](https://developer.zettle.com/docs/api/product-library/user-guides/manage-products/create-products#bulk-import-of-products) section from Zettle documentation.
```
{"developerMessage":"Import limit=10000 items per 24hrs reached for organizationUuid=882c0cfe-8262-11eb-a0fe-f7e66d19bb09","errorType":"IMPORT_LIMIT_REACHED","violations":[]}
```

## IMPORT_SIZE_EXCEEDED
Zettle limits the maximum number of items included in a single request. This limit refers to the number of superproducts; it doesn't count individual subproduct updates.<br/>
See [Bulk import of products](https://developer.zettle.com/docs/api/product-library/user-guides/manage-products/create-products#bulk-import-of-products) section from Zettle documentation.
```
{"developerMessage":"Maximum size for import=2000 items organizationUuid=882c0cfe-8262-11eb-a0fe-f7e66d19bb09","errorType":"IMPORT_SIZE_EXCEEDED","violations":[]}
```

## INTERNAL_SERVER_ERROR: InvalidTypeException
Identifiers generated need to conform with UUIDv1 (refer to [RFC4122](https://www.ietf.org/rfc/rfc4122.txt)).
```
{"developerMessage":"InvalidTypeException: 60d9c496-e176-7839-41be-b976395fdf6a is not a Type 1 (time-based) UUID","errorType":"INTERNAL_SERVER_ERROR","violations":[]}
```

## INSUFFICIENT_SCOPE
Every API request requires of certain scopes to be granted by the merchant when connecting our Zettle integration.<br/>
Required scopes for our use cases are included by default.
```
{"developerMessage":"The request requires higher privileges than provided by the access token. The following scopes are required: ALL:SYSTEM","errorType":"INSUFFICIENT_SCOPE"}
```

## INVALID_CLIENT_JSON
```
{"developerMessage":"Failed to deserialize JSON, Reason=Invalid UUID string: TestUUID (through reference chain: java.util.ArrayList[2]->com.izettle.product.library.core.manager.ImportManager$LeanProduct[\\"uuid\\"])","errorType":"INVALID_CLIENT_JSON","violations":[]}
```

## INVALID_SIZE: size must be between 1 and 99 (variants.options)
Zettle imposes a limit on the number of subproducts per product.
See [Create products](https://developer.zettle.com/docs/api/product-library/user-guides/manage-products/create-products) section from Zettle documentation.<br/>
See also [Product library data relationships](https://developer.zettle.com/docs/api/product-library/concepts/product-library-data#relationships) section.
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations": [{"propertyName":"variants[0].options","developerMessage":"size must be between 1 and 99","constraintType":"INVALID_SIZE","expected":"1-99","value":"[]"},{"developerMessage":"Variants expected to have 1 variant opti...
```

## LOCATION_NOT_FOUND
```
{"developerMessage":"632be7d4-f093-11eb-b143-ca33b7c92053 not found in: [8f59b9b2-828b-11eb-b4b7-4bba58105b03, 8f59b9ee-828b-11eb-b512-f9a20a1c5cc2, 8f59ba16-828b-11eb-8c20-595097492acf, 8f59ba2a-828b-11eb-b4fc-4ed69776b8ac]","errorType":"LOCATION_NOT_FOUND","violations":[]}
```

```
{"developerMessage":"Location not found. UUID=62462ab1-871c-17eb-2065-d580921e244b","errorType":"LOCATION_NOT_FOUND","violations":[]}
```

## MISSING_ETAG
Zettle can optimize product update operations using something called ETag.<br/>
See [Fetching and updating proucts](https://developer.zettle.com/docs/api/product-library/concepts/product-library-management#fetching-and-updating-products) section from Zettle documentation.<br/>
See also [Update a single product](https://developer.zettle.com/docs/api/product-library/user-guides/manage-products/update-products#update-a-single-product) section.
```
{"developerMessage":"If-Match header is required","errorType":"MISSING_ETAG","violations":[]}
```

## SUBSCRIPTION_NOT_FOUND
```
{"developerMessage":"SubscriptionUuid 63eb91bd-10a7-1f26-2672-d580921ea625 was not found","errorType":"SUBSCRIPTION_NOT_FOUND","violations":[]}
```

## UNKNOWN/null

### UNKNOWN/null: Application connection not found
```
{"developerMessage":"Application connection for organization=1a52e2a8-824d-11eb-acc8-aafa2648b59f connecting to application=c37effb9-ddfe-4ead-9d25-905d7d2dcc81 not found","errorType":null,"violations":[]}
```

### UNKNOWN/null: Cannot deserialize value of type CurrencyId from String
CurrencyId field value must match one of the enums listed for request.variants.price.currencyId in [updateProduct](https://developer.zettle.com/docs/api/product-library/reference#updateProduct) request.
```
{"developerMessage":"Failed to deserialize JSON. Type=ProductCreateRequest, Reason=Cannot deserialize value of type `com.izettle.java.CurrencyId` from String \"GB\": not one of the values accepted for Enum class: [FJD, LVL, MXN, BBD, CDF, HNL, CLP, TND, ZAR, CUC, BSD, SDG, IQD, CUP, TWD, RSD, KMF, DOP, MYR, FKP,...
```

### UNKNOWN/null: cannot deserialize value of type EventName from String
```
{"developerMessage":"Cannot deserialize value of type `com.izettle.pusher.api.dto.EventName` from String \"[\": [ was not one of [PurchaseCreated, InvoiceCreated, OrganizationUpdated, OrganizationFeatureUpdated, PaymentInitiated, PaymentCreated, PaymentCanceled, CardPaymentAuthorized, CardPaymentInvalid, CardStoredMessage, RiskE
```

### UNKNOWN/null: collection can not contain null values
```
{"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"developerMessage":"Collection can not contain null values","constraintType":"UNKNOWN","value":"[null]"}]}
```

### UNKNOWN/null: could not connect to HOST
Following error was due to an invalid/expired certificate for ePages server.
```
{"developerMessage":"Could not connect to https://zettle-es.zettle-01.epages.systems/epages/zettle-es.admin/?ChangeAction=ZettleStockCallback&ObjectPath=/Shops/zettle-es/Zettle&ValidationToken=NGU1Njk0M2I1Y2I5YTU5OWMxMTM0NmM5ODBmYTk4Zjk3MTg3NDc1MzcxNzllM2IyMzlkNTE2NjNhOTAxYmYxMF8xNjU0NTA3MjQ4, PKIX path validation failed: java.security.cert.CertPathValidatorException: validity check failed","errorTy...
```

### UNKNOWN/null: failed to deserialize JSON (not a valid 'long' value)
```
{"developerMessage":"Failed to deserialize JSON. Type=ProductCreateRequest, Reason=Cannot deserialize value of type `long` from String \"92.4\": not a valid `long` value\n at [Source: (String)\"{\"externalReference\":\"24721\",\"name\":\"[Example] Upango Test 001\",\"description\":\"<p>Test description</p>\",\"u...
```

### UNKNOWN/null: instantiation of CreateSubscriptionRequest value failed
Field `contactEmail` is compulsory when creating a subscription to Zettle webhooks.
```
{"developerMessage":"Instantiation of [simple type, class com.izettle.pusher.api.request.CreateSubscriptionRequest] value failed for JSON property contactEmail due to missing (therefore NULL) value for creator parameter contactEmail which is a non-nullable type\n at [Source: (org.glassfish.jersey.message.internal.ReaderIntercept...
```

### UNKNOWN/null: invalid_grant
Zettle response for an invalid refresh token, which returns a 400 HTTP code.<br>
Merchant will have to go through authorization process ([OAuth - code grant](https://developer.zettle.com/docs/api/oauth/api-reference#authorisation-code-grant)) again.<br/>
Response body looks like follows:
```
{ "error": "invalid_grant", "error_description": "invalid refresh_token" }
```

### UNKNOWN/null: invalid json
```
{"developerMessage":"Invalid json. Jackson says: Cannot deserialize value of type `com.izettle.inventory.api.shared.TrackingStatusChange` from String \"57fffda7-03fc-11ec-ad5e-7992db7b556c\": 57fffda7-03fc-11ec-ad5e-7992db7b556c was not one of [START_TRACKING, STOP_TRACKING, NO_CHANGE]","errorType":null,"violations":[]}
```

### UNKNOWN/null: method not allowed
```
{"developerMessage":"HTTP 405 Method Not Allowed","errorType":null,"violations":[]}
```

### UNKNOWN/null: service is at capacity
Following error could indicate that image service is overloaded and we need to wait in order to continue exporting product images.
```
{"developerMessage":"Service is at capacity and is currently unable to process the request, please try again later.","errorType":null,"violations":[]}
```

## UNPARSABLE_ENTITY
```
{"developerMessage":"Invalid input:Invalid UUID string: 0xk\u0011\u0011<85><82>\u0001\u0010=<85>r", "errorType":"UNPARSABLE_ENTITY","violations":[]}
```

## VAT_MISSING_ON_VAT_USER
Field `vatPercentage` is mandatory when Zettle account is configured to use VAT.
```
{"developerMessage":"VAT is mandatory. 2 products without VAT","errorType":"VAT_MISSING_ON_VAT_USER","violations":[]}
```

## VAT_NOT_ALLOWED_IN_COUNTRY
Value of field `vatPercentage` must match any of the allowedVats by Zettle. Those `allowedVats` are preconfigured in Zettle by country and we cannot change them.<br/>
See "Note" in [Create a product](https://developer.zettle.com/docs/api/product-library/user-guides/manage-products/create-products#create-a-product) section from Zettle documentation.<br/>
If merchant were to adjust tax matrix it may result in the following error.
```
{"developerMessage":"VAT values=[33] is not allowed in country=GB, allowedVats=[20, 12.5, 5, 0]","errorType":"VAT_NOT_ALLOWED_IN_COUNTRY","violations":[]}
```

```
{"developerMessage":"VAT values=[33] is not allowed in country=DE, allowedVats=[19, 16, 10.7, 9.5, 7, 5, 0]","errorType":"VAT_NOT_ALLOWED_IN_COUNTRY","violations":[]}
```

```
{"developerMessage":"VAT values=[5] is not allowed in country=ES, allowedVats=[21, 10, 8, 7, 6.5, 4, 0]","errorType": "VAT_NOT_ALLOWED_IN_COUNTRY","violations":[]}
```
