# CONTENT
This document contains relevant information regarding Zettle API imposing a limit on the number of
variations per product.

## Variations per product API limit
From [(1)][Product library - Recommendations and limits]</a>:
> You can only assign up to 99 variants to a product

Same limit is stated in [(2)][Product library - Relationships]:
> A product contains maximum 99 variants.

Contrary to what we read in API error response:
> "developerMessage": "Invalid variant size. Max: 300"

However, we clarified with Zettle and they said we should stick to the <strong>99 variants limit</strong>.

## Behaviour of integration
Our Zettle integration will enforce API limits and we'll do a best effort when merchant doesn't comply with the requirements.

The behaviour for the different cases is the following:
- [(a)][Product library - Recommendations and limits] If a product is created with more variations than allowed by Zettle API, product won't be exported to Zettle.
- [(b)][Product library - Recommendations and limits] If a product was previously exported to Zettle and merchant adds a number of variations that go past the limit then all these new variations won't be exported to Zettle.
  However, any subsequent modifications in any of the variations that were already exported will be synced to Zettle.
- [(c)][Product library - Recommendations and limits] If any product reaches a state where number of variations is over the limit, the only thing the merchant can do to export the product is reduce the number of variations under the limit.<br/>
Once number of variations comply with API limits, product and all of its variations will be processed and sent to Zettle in the following export.

Export error feedback was not considered for the MVP so merchant will not be informed when a partial export fails.<br/>
The only way to find out is comparing products in the shop with products in Zettle.

# References
Zettle API developer documentation:
  - [Product library - Recommendations and limits]
  - [Product library - Relationships]

[Product library - Recommendations and limits]: https://developer.zettle.com/docs/api/product-library/concepts/product-library-concept#recommendations-and-limits
[Product library - Relationships]: https://developer.zettle.com/docs/api/product-library/concepts/product-library-data#relationships