# CONTENTS
This document describes the `zettleproductexport` table and explains the reasons why its required.

As stated in [Zettle API documentation][Bulk import of products] export should include `up to 2.000 products per request with less than 10.000 daily changes`.<br/>
This means that we need to export products in batches, that is, we divide the total number of products in single export calls that do not exceed API limits so that large shops will be able to synchronize product library to Zettle.
`zettleproductexport` table may not have the best of names but it doesn't store per-product information, it stores the status of each of the batch export operations against Zettle.

Zettle response for an export operation ([[1]][Bulk import of products], [[2]][Import library items], [[3]][Get status for an import]) will provide us, among other information, the UUID associated to the export (a unique identifier as specified in [RFC4122](https://www.ietf.org/rfc/rfc4122.txt); i.e: `8a228d81-b12f-11ed-9d98-ff0c068ae81c`).<br/>
We need to keep track of each of the batch exports since the global status is calculated as the combination of batch statuses according to the following rules:
- If there is any RUNNING batch --> GlobalStatus=RUNNING.
- If there is no RUNNING batch and there is at least one FAILED batch --> GlobalStatus=FAILED.
- If all batches are in COMPLETED status --> GlobalStatus=COMPLETED.

We create a random generated `GlobalExportUUID` which we'll use to search the DB and retrieve individual batch exports that are associated to that GlobalExportUUID.

# Table columns

The table columns are defined as follows:
- `globalexportuuid` stores the random generated UUID (v4) (part of the primary key).
- `batchnumber` is an integer that tells the position of the batch inside global export (part of the primary key).
- `siteid` is the shop ID which will be used to retrieve export data from a certain shop (useful for debugging).
- `exporttype` indicates whether it's an INITIAL_EXPORT (value 1), PRODUCT_UPDATE (value 2) or PRICE_UPDATE (value 3). Refer to `EXPORT_TYPE` in [Constants.pm](/DE_EPAGES/Zettle/API/Constants.pm).
- `itemcount` is the number of items included in the batch. The first value it gets when entry is created is a rough estimate that counts subproducts. When export completes successfully value is properly updated to item count returned by Zettle (which only counts superproducts).
- `batchexportstatus` is a number that tells the status of this batch export. Refer to `INITIAL_EXPORT_STATUS` in [Constants.pm](/DE_EPAGES/Zettle/API/Constants.pm).
- `batchexportuuid` is the UUID (v1) returned by Zettle for an export operation. It won't be available until we get a response from Zettle.<br/>
  Even if we get a response, it might not contain UUID if there was a general export error and no item could be processed.
- `startedon` is datetime when batch export was executed.
- `updatedon` is the last time batch export data was updated.
- `errortype` is a short identifier in case there's a general error during export.
- `errormessage` is a detailed description of the error. Gives us additional information for debugging purposes.

# Table cleanup and size considerations
In order to keep `zettleproductexport` table under control a scheduler task is added. This task will remove entries from the table for batches older than the specified number of days (default: 30 days). See `RemoveZettleProductExportData` section in [Scheduler.conf](/DE_EPAGES/Zettle/Data/Scheduler/Scheduler.conf). Table column used for this comparison is `startedon`.<br/>
Cleanup script is [removeZettleProductExportData.pl](/DE_EPAGES/Zettle/Scripts/removeZettleProductExportData.pl).

Entries will be added to the table every time an initial export or an incremental export operation is run for a shop. The number of entries added for a single operation will depend on the number of products to be exported taking into account that a batch export can include as many items as stated API limit [[1]][Bulk import of products], meaning 2.000 as of Feb 2023.

There are two types of operations: scheduled exports and manual exports.
- The first time a merchant sets up the integration he will run an initial export that will export all shop products to Zettle. After the initial export, manual incremental exports are not expected to be run too often.
- The typical case is that the merchant relies on the scheduled export. For every shop connected to Zettle a nightly job will run an incremental export. For each shop, the export frequency depends on merchant configuration and can vary from daily to monthly.

Let's consider all merchants go for the daily update. This would result in an incremental export every night for all shops connected to Zettle. Even for merchants with more than 2.000 products in the shop, this incremental update only processes products that were modified since last export. We can safely assume that only a handful of products will be updated between two consecutive exports.

We can infer that the number of new daily entries in the table will be a direct proportion to the number of merchants connected to Zettle. Manual updates will also generate entries but the goal of the integration is to work seamlessly so we only expect a few manual updates per merchant.

If the table size proves to grow too much in production environments _RemoveZettleProductExportData_ scheduler entry could be adjusted to preserve entries for 15 days intead of 30, for instance.

# References
- [Bulk import of products]
- API reference:
  - [Import library items]
  - [Get status for an import]

[Bulk import of products]: https://developer.zettle.com/docs/api/product-library/user-guides/manage-products/create-products#bulk-import-of-products
[Import library items]: https://developer.zettle.com/docs/api/product-library/reference#importLibraryV2
[Get status for an import]: https://developer.zettle.com/docs/api/product-library/reference#getStatusByUuid